@@ -129,6 +129,7 @@ impl Client {
     ) -> Result<(EntryHash, RegisterWriteAheadLog), Error> {
         // First we fetch it so we can get the causality info,
         // either from local CRDT replica or from the network if not found
+        debug!("Writing to register at {:?}", address);
         let mut register = self.get_register(address).await?;
 
         // We can now write the entry to the Register
@@ -163,7 +164,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::GetRegister((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -179,7 +180,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::ReadRegister((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -196,7 +197,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::GetRegisterEntry((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -213,7 +214,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::GetRegisterOwner((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -234,7 +235,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::GetRegisterUserPermissions((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -247,7 +248,7 @@ impl Client {
         let query_result = self.send_query(query).await?;
         match query_result.response {
             QueryResponse::GetRegisterPolicy((res, op_id)) => {
-                res.map_err(|err| Error::ErrorMessage { source: err, op_id })
+                res.map_err(|err| Error::ErrorMsg { source: err, op_id })
             }
             _ => Err(Error::ReceivedUnexpectedEvent),
         }
@@ -281,7 +282,7 @@ mod tests {
         },
         Error,
     };
-    use crate::messaging::data::Error as ErrorMessage;
+    use crate::messaging::data::Error as ErrorMsg;
     use crate::retry_loop_for_pattern;
     use crate::types::{
         log_markers::LogMarker,
@@ -534,8 +535,8 @@ mod tests {
             .await
         {
             Ok(_) => bail!("Should not be able to retrieve an entry for a random user"),
-            Err(Error::ErrorMessage {
-                source: ErrorMessage::NoSuchEntry,
+            Err(Error::ErrorMsg {
+                source: ErrorMsg::NoSuchEntry,
                 ..
             }) => Ok(()),
             Err(err) => Err(eyre!(
@@ -589,8 +590,8 @@ mod tests {
             .await
         {
             Ok(_) => bail!("Should not be able to retrieve an entry for a random user"),
-            Err(Error::ErrorMessage {
-                source: ErrorMessage::NoSuchEntry,
+            Err(Error::ErrorMsg {
+                source: ErrorMsg::NoSuchEntry,
                 ..
             }) => Ok(()),
             Err(err) => Err(eyre!(
@@ -686,8 +687,8 @@ mod tests {
             .instrument(tracing::info_span!("final get"))
             .await
         {
-            Err(Error::ErrorMessage {
-                source: ErrorMessage::NoSuchEntry,
+            Err(Error::ErrorMsg {
+                source: ErrorMsg::NoSuchEntry,
                 ..
             }) => Ok(()),
             Err(err) => Err(eyre!(
@@ -796,7 +797,7 @@ mod tests {
         let batch2 = client.delete_register(address).await?;
         match client.publish_register_ops(batch2).await {
             Err(Error::ErrorCmd {
-                source: ErrorMessage::InvalidOperation(_),
+                source: ErrorMsg::InvalidOperation(_),
                 ..
             }) => {}
             Err(err) => bail!(
