@@ -6,12 +6,14 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
+mod chunk_store;
 mod encoding;
 mod errors;
 mod event_store;
 mod lru_cache;
 mod used_space;
 
+pub(crate) use chunk_store::ChunkStore;
 pub(crate) use encoding::{deserialise, serialise};
 pub(crate) use errors::{convert_to_error_msg, Error, Result};
 pub(crate) use event_store::EventStore;
